#include <Arduino.h>
#include "buzzer.h"
#include <pfodDelay.h>

extern void initBuzzer(); // sets output pin
extern void buzzer_On();
extern void buzzer_Off();

pfodDelay buzzerTimer;
const unsigned long BUZZER_TIMER_INTERVAL = 100; // 0.1 sec

buzzerTypeEnum buzzerType = BUZZER_OFF;
const int ERROR_BUZZER_PERIOD = 2; // 0.2sec
const int ERROR_FLASH_OFF = 1; // 50% buzzer
int buzzerCounter = 0;
int errorBuzzerCounter = 0;
int buzzerOff = 1;
int buzzerPeriod = 1;
bool buzzerOn = false;

void setBuzzerOn() {
  buzzerOn = true;
  buzzerCounter = 0;
  buzzer_On();
}

void setBuzzerOff() {
  buzzerOn = false;
  buzzerCounter = 0;
  buzzer_Off();
}


/**
   Set buzzer type
*/
void setBuzzer(buzzerTypeEnum b_type) {
  if (b_type == BUZZER_OFF)  {
    buzzerType = b_type;
    setBuzzerOff();
    return;
  }
  if (b_type == buzzerType) {
    return; // no chabnge
  }
  buzzerCounter = 0;
  if (b_type == BUZZER_CLOSED) {
    buzzerType = b_type;
    setBuzzerOff();
  } else if (b_type == BUZZER_OPEN) {
    buzzerType = b_type;
    setBuzzerOff();
  } else if (b_type > buzzerType)  {
    setBuzzerOff();
    buzzerType = b_type;
    switch (buzzerType) {
      case BUZZER_ERROR:
        buzzerPeriod = ERROR_BUZZER_PERIOD;
        buzzerOff = ERROR_FLASH_OFF;
        errorBuzzerCounter = 0;
        break;
      case BUZZER_MSG_RETRY:
        setBuzzerOff(); // clears counters
        setBuzzerOn();
        break;
      default:
        break;
    }
  }
}

void buzzerInit() {
  initBuzzer();
  setBuzzerOff();
  setBuzzer(BUZZER_OFF);
  buzzerTimer.start(BUZZER_TIMER_INTERVAL);
}

void buzzerToggle() {
  buzzerOn = !buzzerOn;
  if (buzzerOn) {
    buzzer_On();
  } else {
    buzzer_Off();
  }
}

void handleBuzzer() {
  if (buzzerType == BUZZER_OFF) {
    setBuzzerOff();
  } else {
    // else
    if (buzzerType == BUZZER_CLOSED) {
      if (buzzerCounter >= 19) {
        setBuzzer(BUZZER_OFF);
        return;
      }
      //  OFF 0        ON  2               OFF  6                    ON   2                   OFF    6               ON     2                   OFF 6                        ON 1                  OFF
      if ((buzzerCounter == 0) || (buzzerCounter == 2) || (buzzerCounter == 8) || (buzzerCounter == 10) || (buzzerCounter == 16) || (buzzerCounter == 18)) {// || (buzzerCounter == 33)) {
        buzzerToggle();
      }
    } else if (buzzerType == BUZZER_OPEN) {
      if (buzzerCounter >= 35) {
        setBuzzer(BUZZER_OFF);
        return;
      }
      //  OFF 0        ON  13               OFF  8                    ON   13                   OFF   8              ON     8                   OFF 2                        ON 12                  OFF
      if ((buzzerCounter == 0) || (buzzerCounter == 13) || (buzzerCounter == 21) || (buzzerCounter == 34)) { //|| (buzzerCounter == 40) || (buzzerCounter == 20) || (buzzerCounter == 22)) {
        buzzerToggle();
      }
    } else if (buzzerType == BUZZER_NEEDS_CHARGING) {
      if (buzzerCounter >= 55) {
        setBuzzer(BUZZER_OFF);
        return;
      }
      //  OFF 0        ON  2               OFF  4                    ON   8                   OFF   6              ON     2                   OFF 4                        ON 8                  OFF 6
      if ((buzzerCounter == 0) || (buzzerCounter == 2) || (buzzerCounter == 6) || (buzzerCounter == 14) || (buzzerCounter == 20) || (buzzerCounter == 22) || (buzzerCounter == 26) || (buzzerCounter == 34)
          //        ON     2                   OFF 4                      ON 8
          || (buzzerCounter == 40)  || (buzzerCounter == 42)  || (buzzerCounter == 46) || (buzzerCounter == 54)) {
        buzzerToggle();
      }

    } else if (buzzerType == BUZZER_ERROR) {
      // buzzer
      errorBuzzerCounter++;
      if (errorBuzzerCounter >= buzzerPeriod) {
        errorBuzzerCounter = 0; // reset
        setBuzzerOff();
      } else if (errorBuzzerCounter == buzzerOff) {
        buzzerToggle();
      }
    } else if (buzzerType == BUZZER_MSG_RETRY) {
      if (buzzerCounter >= 1) {
        setBuzzer(BUZZER_OFF);
        return;
      }
    }
    buzzerCounter++;
  }
}

void checkBuzzer() {
  if (buzzerTimer.justFinished()) {
    buzzerTimer.repeat(); // prevents drift
    handleBuzzer();
  }
}
