#include <Arduino.h>
#include "flasher.h"
#include <pfodDelay.h>

extern void initLed();
extern void led_On();
extern void led_Off();

pfodDelay ledFlasherTimer;
const unsigned long FLASH_TIMER_INTERVAL = 100; // 0.1 sec

bool flashOn = false;

flashTypeEnum flashType = FLASH_OFF;
const int FAST_FLASH_PERIOD = 2; // 0.2sec
const int FAST_FLASH_OFF = 1; // 50% flash
const int SLOW_FLASH_PERIOD = 18; // 1.8sec
const int SLOW_FLASH_OFF = 9; // 50% flash
int flashCounter = 0;
int flashOff = 1;
int flashPeriod = 1;

void setFlash(flashTypeEnum f_type);

void flashInit() {
  initLed();
  led_On(); // ON
  setFlash(FLASH_OFF);
  ledFlasherTimer.start(FLASH_TIMER_INTERVAL);
}

void turnFlashOn() {
  flashOn = true;
  led_On();
}

void turnFlashOff() {
  flashOn = false;
  led_Off();
}

void toggleFlash() {
  flashOn = !flashOn;
  if (flashOn) {
    led_On();
  } else {
    led_Off();
  }
}

/**
   Set flash type
*/
void setFlash(flashTypeEnum f_type) {
  if (f_type == FLASH_OFF)  {
    flashType = f_type;
    turnFlashOff();
    return;
  }
  if (f_type == NO_FLASH)  {
    flashType = f_type;
    turnFlashOn();
    return;
  }
  if (f_type == flashType) {
    return; // no chabnge
  }
   flashCounter = 0;
  if (f_type == CHARGING_FLASH) {
    flashType = f_type;
    turnFlashOn();
  } else if (f_type > flashType)  {
    turnFlashOn();
    //up the flash speed, the above test ensures the powerOffTimer only reset once on switching to Fast Flash
    flashType = f_type;
    switch (flashType) {
      case FAST_FLASH:
        //       powerOffTimer = millis(); // reset power off time so will flash for full 15sec if error
        flashPeriod = FAST_FLASH_PERIOD;
        flashOff = FAST_FLASH_OFF;
        break;
      case SLOW_FLASH:
        flashPeriod = SLOW_FLASH_PERIOD;
        flashOff = SLOW_FLASH_OFF;
        break;
      default:
        break;
    }
  }
}

void handleFlash() {
  if (flashType == FLASH_OFF) {
    turnFlashOff(); // just turn led on
  } else if (flashType == NO_FLASH) {
    turnFlashOn(); // just turn led on
  } else if (flashType == CHARGING_FLASH) {
    flashCounter++;
    if (flashCounter >= 24) {
         flashCounter = 0;
      turnFlashOn();
      //           OFF                    ON                         OFF                   0N                      OFF                         0N                   OFF
    } else if ((flashCounter == 4) || (flashCounter == 6) || (flashCounter == 14) || (flashCounter == 15) || (flashCounter == 16) || (flashCounter == 17) || (flashCounter == 18)) {
      toggleFlash();
    }
  } else {
    // flash
    flashCounter++;
    if (flashCounter >= flashPeriod) {
      turnFlashOn();
      flashCounter = 0;

    } else if (flashCounter == flashOff) {
      toggleFlash();
    }
  }
}

void checkLedFlasher() {
  if (ledFlasherTimer.justFinished()) {
    ledFlasherTimer.repeat(); // prevents drift
    handleFlash();
  }
}
